<?php
namespace PrestaShop\Module\PolarisPrestaConnector;

use Exception;
use PolarisPrestaConnector;
use PrestaShop\Module\PolarisPrestaConnector\Models\ProductRef;
use ProductSupplier;

/**
 * Configuration du module
 */
class Syncer
{
    /**
     * Module
     */
    public PolarisPrestaConnector $module;

    /**
     * Indique si nous sommes en mode CRON
     */
    private $isCron = false;

    /**
     * Pont vers le backoffice
     */
    public BridgeAdapter $bridge;

    /**
     * Lien vers le fichier d'audit
     */
    private mixed $auditFileHandler = null;

    /**
     * Cache de références fournisseur
     * null quand pas encore chargé
     * 
     * @var SupplierRefManager
     */
    private ?SupplierRefManager $refMngr = null;

    /**
     * Instanciation
     * 
     * @param PolarisPrestaConnector $module Module
     * @param bool $isCron Indique si nous sommes en mode CRON
     */
    public function __construct(PolarisPrestaConnector $module, $isCron = false)
    {
        // Déclenche la mise à jour de la base si besoin
        $module->dbUpgrade();

        $this->module = $module;        
        $this->bridge = $module->getBridge();
        $this->isCron = $isCron;
    }

    /**
     * Obtient le nom du verrou
     * 
     */
    public function getLockName() : string
    {
        return 'syncer-' . $this->module->name . '-' . str_replace('.', '_', $_SERVER['HTTP_HOST']);
    }

    /**
     * Obtient le gestionnaire de références fournisseur
     * 
     * @return SupplierRefManager
     */
    public function getReferencesManager() : SupplierRefManager
    {
        if (is_null($this->refMngr))
            $this->refMngr = new SupplierRefManager($this->module->getCfg()->get(SyncConfiguration::CFG_SUPPLIER));
        return $this->refMngr;
    }

    /**
     * Resynchronise un seul produit particulier
     * 
     */
    public function syncProduct($id_product)
    {
        // On précharge certaines données de configuration que nous allons qui nous seront utiles par la suite
        $this->module->getCfg()->preload([
            SyncConfiguration::CFG_SUPPLIER,
            SyncConfiguration::CFG_ACTIVE,
            SyncConfiguration::CFG_WORKMODE,
        ]);

        // Si le module n'est pas actif : pas de synchro !
        if (!$this->module->getCfg()->get(SyncConfiguration::CFG_ACTIVE))
            throw new \Exception('La synchronisation désactivée.');
 
        // on regarde sur la base de données si nous avons un traitement déjà en cours
        $db = \Db::getInstance();

        // on va passer par un verrou pour éviter de lancer plusieurs fois le traitement
        // le nom du verrou est composé du nom du module et du nom de domaine du site
        $lockName = $this->getLockName();

        // on tente de prendre le verrou
        if (!$db->execute('SELECT GET_LOCK(\'' . $lockName . '\', 30)'))
            // le verrou est déjà pris, on ne fait rien
            throw new \Exception("Une synchronisation est déjà en cours !");

        try
        {
            // On va chercher les références fournisseurs
            $refMngr = new SupplierRefManager($this->module->getCfg()->get(SyncConfiguration::CFG_SUPPLIER), [$id_product]);
            $productRef = $refMngr->translateProductId($id_product);
            if (!$productRef)
                throw new \Exception("Produit non géré par le backoffice " . $this->bridge->getId());
            
            $syncerProducts = new SyncerProducts($this);            
            $syncerProducts->syncProducts(
                $this->module->getBridge()->pullProducts($productRef)
            );
        }
        finally
        {
            // on libère le verrou
            // et si on y arrive pas, on le mentionne dans le journal de Prestashop
            if (!$db->execute('SELECT RELEASE_LOCK(\'' . $lockName . '\')'))
                \PrestaShopLogger::addLog('Impossible de libérer le verrou ' . $lockName, 3);
        }
    }
    

    /**
     * Synchronise les données
     */
    public function sync()
    {
        // On précharge certaines données de configuration que nous allons qui nous seront utiles par la suite
        $this->module->getCfg()->preload([
            SyncConfiguration::CFG_SUPPLIER,
            SyncConfiguration::CFG_ACTIVE,
            SyncConfiguration::CFG_WORKMODE,
        ]);
        
        // Si le module n'est pas actif : pas de synchro !
        if (!$this->module->getCfg()->get(SyncConfiguration::CFG_ACTIVE))
        {
            $this->audit('[WRN] La synchronisation désactivée.');
            return;
        }

        // on regarde sur la base de données si nous avons un traitement déjà en cours
        $db = \Db::getInstance();

        // on va passer par un verrou pour éviter de lancer plusieurs fois le traitement
        // le nom du verrou est composé du nom du module et du nom de domaine du site
        $lockName = $this->getLockName();

        // on tente de prendre le verrou
        if (!$db->execute('SELECT GET_LOCK(\'' . $lockName . '\', 0)')) {
            // le verrou est déjà pris, on ne fait rien
            echo "Traitement déjà en cours !";
            return;
        }        

        try
        {
            // Etape préliminaire : tâches quotidiennes !
            // Toutes les 24h, on regarde s'il n'y a pas une mise à jour du module
            $lastCheck = \Configuration::get($this->module->name . '_last_update_check');
            if (!$lastCheck || (time() - $lastCheck) > 86400)
            {
                // On enregistre la date de la dernière vérification
                \Configuration::updateValue($this->module->name . '_last_update_check', time());

                $updater = new AutoUpdater($this->module);
                $updater->searchUpdate();
            }

            // TEST : on remet à zéro la position de synchronisation
            // $this->module->getBridge()->resetSyncPosition();
            
            // Ensuite, on regarde si dans la configuration, nous avons l'identifiant du fournisseur pour ce module
            $supplierId = $this->module->getCfg()->get(SyncConfiguration::CFG_SUPPLIER);
            if ($supplierId)
            {
                // Chargement du fournisseur
                $supplier = new \Supplier($supplierId);
                if ($supplier->id != $supplierId)
                {
                    // On log dans prestashop
                    \PrestaShopLogger::addLog('Impossible de charger le fournisseur de référence #' . $supplierId . ' => recréation ! ' . $supplier->name, 3, null, "supplier", $supplierId);
                    $supplierId = 0;
                }
            }
            if (!$supplierId)
            {
                // Autrement, on le crée automatiquement
                $supplier = new \Supplier();
                // Le nom du fournisseur est le nom qui précède "PolarisPrestaConnector" dans le nom du module, avec une majuscule                
                $supplier->name = ucfirst(substr($this->module->name, 0, strpos($this->module->name, 'PolarisPrestaConnector')));
                if ($supplier->name == '')
                    $supplier->name = 'DEV';
                $supplier->add();
                if (!$supplier->id)
                    throw new Exception('Impossible de créer le fournisseur de référence !');
                $this->module->getCfg()->set(SyncConfiguration::CFG_SUPPLIER, $supplier->id);
                // On journalise la création du fournisseur dans les journaux de Prestashop
                \PrestaShopLogger::addLog('Création du fournisseur ' . $supplier->name, 1, null, "supplier", $supplier->id);
            }                
            
            // Etape 1 : on transmet les commandes
            $syncerOrders = new SyncerOrders($this);
            $syncerOrders->sync();
            $this->module->getBridge()->resetSyncPosition();
            // Etape 2 : on aspire le catalogue si workmode 1
            if ($this->module->getCfg()->get(SyncConfiguration::CFG_WORKMODE) == 1)
            {                                
                $syncerProducts = new SyncerProducts($this);
                $syncerProducts->sync();
            }

            // Etape 3 : on aspire les stocks et les prix
            $syncerStocks = new SyncerStocks($this);
            $syncerStocks->sync();

            // Etape 4 : les promotions ?
            $syncerDiscounts = new SyncerDiscounts($this);
            $syncerDiscounts->sync();

            // Etape 5 : les bons de réduction ?            
            $syncerVouchers = new SyncerVouchers($this);
            $syncerVouchers->sync();

            // Fin !
            $this->audit('[INFO] Synchronisation terminée');
        }
        catch (Exception $e)
        {
            // Journalisation de l'erreur
            \PrestaShopLogger::addLog('Erreur à la synchronisation : '.$e->getMessage(), 3, null, $this->module->name); 
            $this->audit('[FATAL] ' . $e->getMessage());
        }
        finally
        {
            // on libère le verrou
            // et si on y arrive pas, on le mentionne dans le journal de Prestashop
            if (!$db->execute('SELECT RELEASE_LOCK(\'' . $lockName . '\')'))
                \PrestaShopLogger::addLog('Impossible de libérer le verrou ' . $lockName, 3);
        }
    }

    /**
     * Audit des actions
     * 
     * @param string $message Message à journaliser     
     */
    public function audit(string $message)
    {
        if (!$message)
            return; // rien à faire 

        // Journalisation de l'audit, séparé, dans un fichier à part
        // le fichier est il ouvert ?
        if (!$this->auditFileHandler)
        {
            // sinon, on l'ouvre en mode ajout dans la racine /var/audits
            // si le répertoire n'existe pas, on le crée            
            $path = _PS_ROOT_DIR_ . '/var/audits/'.$this->module->name;
            if (!is_dir($path))
                mkdir($path, 0777, true);

            // On supprime les fichiers d'audit qui ont plus de 60 jours
            $files = scandir($path);
            foreach ($files as $file)
            {
                if (is_file($path . '/' . $file) && (time() - filemtime($path . '/' . $file)) > 60 * 86400)
                    unlink($path . '/' . $file);
            }

            // Le fichier est nommé audit-AAAAMMJJHHmmSS.log
            $this->auditFileHandler = fopen($path . '/audit-' . date('Ymd') . '.log', 'a');
            if (!$this->auditFileHandler)
                throw new Exception('Impossible d\'ouvrir le fichier d\'audit');            
        
            fwrite($this->auditFileHandler, date('Y-m-d H:i:s') . ' - Nouvelle synchronisation'."\n");
        }

        // On écrit le message dans le fichier
        fwrite($this->auditFileHandler, date('Y-m-d H:i:s') . ' - ' . $message . "\n");

        // Si nous sommes en mode DEBUG, alors on sort le message sur la sortie standard si nous sommes un tâche CRON
        if (_PS_MODE_DEV_ && $this->isCron)
            echo $message . "\n"; 
    }    
}